////////////////////////////////////////////////////////////
//
// SFML - Simple and Fast Multimedia Library
// Copyright (C) 2007-2009 Laurent Gomila (laurent.gom@gmail.com)
//
// This software is provided 'as-is', without any express or implied warranty.
// In no event will the authors be held liable for any damages arising from the use of this software.
//
// Permission is granted to anyone to use this software for any purpose,
// including commercial applications, and to alter it and redistribute it freely,
// subject to the following restrictions:
//
// 1. The origin of this software must not be misrepresented;
//    you must not claim that you wrote the original software.
//    If you use this software in a product, an acknowledgment
//    in the product documentation would be appreciated but is not required.
//
// 2. Altered source versions must be plainly marked as such,
//    and must not be misrepresented as being the original software.
//
// 3. This notice may not be removed or altered from any source distribution.
//
////////////////////////////////////////////////////////////

#ifndef SFML_DISPLAYREF_HPP
#define SFML_DISPLAYREF_HPP

////////////////////////////////////////////////////////////
// Headers
////////////////////////////////////////////////////////////
#include <X11/Xlib.h>


namespace sf
{
namespace priv
{
////////////////////////////////////////////////////////////
/// Provides an access to the display (connection to the X server)
////////////////////////////////////////////////////////////
class DisplayRef
{
public :

    ////////////////////////////////////////////////////////////
    /// Default constructor
    ///
    ////////////////////////////////////////////////////////////
    DisplayRef();

    ////////////////////////////////////////////////////////////
    /// Copy constructor
    ///
    /// \param Copy : Instance to copy
    ///
    ////////////////////////////////////////////////////////////
    DisplayRef(const DisplayRef& Copy);

    ////////////////////////////////////////////////////////////
    /// Destructor
    ///
    ////////////////////////////////////////////////////////////
    ~DisplayRef();

    ////////////////////////////////////////////////////////////
    /// Get the current display
    ///
    /// \return Pointer to the current display
    ///
    ////////////////////////////////////////////////////////////
    ::Display* GetDisplay() const;

    ////////////////////////////////////////////////////////////
    /// Get the input method associated to the display
    ///
    /// \return Input method object
    ///
    ////////////////////////////////////////////////////////////
    XIM GetInputMethod() const;
};

} // namespace priv

} // namespace sf

#endif // SFML_DISPLAYREF_HPP
